clear
set(0,'DefaultFigureWindowStyle','docked')

%% 1. Mesh and conductivity

geom = @circleg;  % circle geometry
[p,e,t] = initmesh(geom);  %coarse mesh (inverse problem)
[p1,e1,t1] = refinemesh(geom,p,e,t);  % fine mesh (direct problem)

figure
subplot(1,2,1)
pdemesh(p,e,t)
subplot(1,2,2)
pdemesh(p1,e1,t1)

nt = size(t,2); % Number of triangles of coarse mesh
bound = p(:,e(1,:)); %boundary points

% Find centroids and areas of triangles of coarse mesh
x = (p(1,t(1,:))+p(1,t(2,:))+p(1,t(3,:)))/3;
y = (p(2,t(1,:))+p(2,t(2,:))+p(2,t(3,:)))/3;
area = pdetrg(p,t);

% The real conductivity
sigma = coeffunction(p1,t1);
sigmamesh = tri2grid2(p1,t1,sigma,x,y);


%% 2. The direct problem

noise = 0.00; % Noise level, in percentage
N = 23; % Number of input (sinusoidal) currents

g = cell(1,N);
u0x = cell(1,N);
u0y = cell(1,N);
u0bound = cell(1,N);
ubound = cell(1,N);

for n=1:N
    phase = randn; % We add a random phase to the currents
    f = @(theta) sin(n*theta + phase); % sinusoidal current
    g{n} = f(cart2pol(bound(1,:),bound(2,:)));
    neumann = @(p,e,u,time) pdeboundeit(p,e,u,time,f);
    u = assempde(neumann,p1,e1,t1,sigma,0,0); %real potential
    u0 = assempde(neumann,p1,e1,t1,1,0,0); % reference potential
       
    [tempx,tempy] = pdegrad(p1,t1,u0);
    u0x{n} = tri2grid2(p1,t1,tempx,x,y);
    u0y{n} = tri2grid2(p1,t1,tempy,x,y);
    
    %boundary measurements
    for i=1:size(bound,2)
        u0bound{n}(1,i) = tri2grid(p1,t1,u0,bound(1,i),bound(2,i));
        ubound{n}(1,i) = tri2grid(p1,t1,u,bound(1,i),bound(2,i));
    end
    %add noise
    ubound{n} = ubound{n} + noise*randn(size(ubound{n}))*max(abs(ubound{n}));
end

% The sensitivity matrix M and the data X
M = zeros((N)^2,nt);
X = zeros((N)^2,1);

for n=1:N
    for m=1:N
        j = N*(m-1)+n;
        M(j,:) = (u0x{n}.*u0x{m}+u0y{n}.*u0y{m}).*area;
        X(j) = sum(g{n}.*u0bound{m}-ubound{n}.*g{m})/size(bound,2)*2*pi;
    end
end   

figure
subplot(1,2,1)
pdeplot(p,e,t,'xydata',sigmamesh)

break
%% 3. The inverse problem

%Use SVD regularization for the inversion of MA=X
subplot(1,2,2)
[U,S,V] = svd(M);

for cut = 1:190
    Sp=pinv(S);
    Sp(cut:end,cut:end)=0;
    recon = V*Sp*U'*X;
    pdeplot(p,e,t,'xydata',1+recon)
    pause(.1)
    title(cut)
    getframe;
end

% Show the exponential decay of the singular values: severely ill-posed IP
figure
subplot(1,2,1)
plot(diag(S))
title('The singular values of the sensitivity matrix M')
subplot(1,2,2)
semilogy(log(diag(S)))
title('The singular values of the sensitivity matrix M in semilog scale')

break


%% 4. Small inclusions with the projection algorithm

f = @(theta) cos(theta); % Input current, relative to a = [1 0]

neumann = @(p,e,u,time) pdeboundeit(p,e,u,time,f);
ua = assempde(neumann,p1,e1,t1,sigma,0,0); %real potential
u0a = assempde(neumann,p1,e1,t1,1,0,0); % reference potential

% boundary measurements
u0abound = zeros(1,size(bound,2));
uabound = zeros(1,size(bound,2));

for i=1:size(bound,2)
    u0abound(1,i) = tri2grid(p1,t1,u0a,bound(1,i),bound(2,i));
    uabound(1,i) = tri2grid(p1,t1,ua,bound(1,i),bound(2,i));
end

% add noise
uabound = uabound + noise*randn(size(uabound))*max(abs(uabound));

% Find x of the location z of the inclusion
X = -1.5:.01:1.5;
Iw = zeros(2,length(X));

dwdn = zeros(1,size(bound,2));
for  j=1:length(X)
    y = [X(j); -1.2];
    gradw = @(x) -1/(2*pi)/norm(x-y)^2*(x-y);
    for i=1:size(bound,2)
        dwdn(i) = dot(gradw(bound(:,i)),bound(:,i));
    end
    Iw(1,j) = sum(dwdn.*(uabound-u0abound))/size(bound,2)*2*pi;
end
[~,j] = min(abs(Iw(1,:)));  zx=X(j);

% Find y of the location z of the inclusion
for  j=1:length(X)
    y = [1.2; X(j)];
    gradw = @(x) -1/(2*pi)/norm(x-y)^2*(x-y);
    for i=1:size(bound,2)
        dwdn(i) = dot(gradw(bound(:,i)),bound(:,i));
    end
    Iw(2,j) = sum(dwdn.*(uabound-u0abound))/size(bound,2)*2*pi;
end

[~,j]=max(abs(Iw(2,:)));  zy=X(j);

z = [zx;zy] %the location of the inclusion

figure
plot(X,Iw(1,:),X,Iw(2,:),X,zeros(size(X)),'k')
legend('I_w[U](z_1)','I_w[U](z_2)')
set(gca,'fontsize', 14);

y = [1.2; 0];
gradw = @(x) -1/(2*pi)/norm(x-y)^2*(x-y);

for i=1:size(bound,2)
    dwdn(i) = dot(gradw(bound(:,i)),bound(:,i));
end
Iw(2,j) = sum(dwdn.*(uabound-u0abound))/size(bound,2)*2*pi;

% The estimate of the size of the inclusion and of the contrast
inclusion = - Iw(2,j) * pi / dot(y-z,[1 0])*norm(y-z)^2
real_value = pi*.2^2/3